    program

! AltFix7 - How to fix the Alt Key lockup in Clarion built apps. Fixes C5, C5.5, C6 and C7, probably fixes C4 and C2.
! by Carl Barnes, www.carlbarnes.com, carl@carlbarnes.com
! (c) 2011 Carl Barnes.
! You may implement this solution, and use this code in your compiled product.
! You may not use this code in a template you distribute or sell.

! The Menu bar is activated by pressing ALT alone (w/o an accelerator key like Alt+F) or pressing F10 (and F10 + Shift, Ctrl or Alt)
! Under Win 7 ALT alone or F10 locks up a Clarion program. Some testers found it required Outlook or Live Mail.
!   Pressing Alt+Esc will unlock it, then minimize, so somewhat confusing...and most users don't know.
!
! A Template by Marius fixes it by ALERT ALT (1024) and F10 in every child, then doing nothing, it has multiple side effects:
!   Menu lines don't appear, Alt+ access to menus doesn't work, ALRT and KEY F10 no longer activates menu, code generated into every window
!
! My Basic ALT or F10 fix is
!   1. Subclass Frame
!   2. Define a FrameSubclass procedure
!   3. Spot WM_SYSCOMMAND msg with BAND(WParm,fff0h)=SC_KEYMENU and Lparm=0  (that's ALT or F10)
!   3a    return(true) so we take the event and do not let the Clarion RTL get it to active the menu
!   3b.   Or my pref, send a message to drop the first menu (Alt+F typically) so much more visual

! At bottom I added ==Quick-Start Instructions== that will appeal to many..............

!====== How to Fix Your Application ==== Detailed Instructions ============================================
!#--> 1. Open the .App file with your Frame, in the Proc tree find the Frame (Main)
!#--> 2. Right-Click on your Frame procedure and select Source
!#--> 3. Alt+F3 search for "Prop:WndProc". If you find it jump to "== Already Subclassed Fix ==" below for the @1,2,3
!#--> 4. Alt+F3 search for "OPEN(AppFrame" to find where your App Frame is opened. In ABC that's Window.Init.
!#--> 5. Embed Code: To Subclass the frame with a Prop:WndProc reassignment
!#--> 6. Close the Main Source. Switch to Module View. 
!#--> 7. Open the the Module that contains Main. Uncheck "Allow Repopulate". Open Embeds and the Module Data Embed
!#--> 8. Embed Code: To define AppFrame_OrigWndProc LONG as module data
!#--> 9. With Main's Module selected add a new Procedure (open Procedure menu, then click New...)
!#--> 10. Name it "SubClassFrame" and select template type SOURCE
!#-->       Prototype:  (SIGNED _hWnd,UNSIGNED _wMsg,UNSIGNED _wParam,SIGNED _LParam),SIGNED,PASCAL   !Win7AltFix
!#-->       Parameters: (SIGNED _hWnd,UNSIGNED _wMsg,UNSIGNED _wParam,SIGNED _LParam)!!,SIGNED,PASCAL !Win7AltFix
!#--> 11. Embed Code: To define the Subclass procedure code.
!#              Be Sure to do ===>> If &File is not 1st menu, i.e. MENU('&File'), then change Alt1stMenu EQUATE.
!#--> 12. Add Win API Declarations. Open Global, then Embeds, then "Inside the Global Map" embed point
!#--> 13. Embed Code: Win API "Inside the Global Map" declarations

!====== Already Subclassed Fix Your Application ==========================================================
! Then you should already have a Subclass procedure defined, hopefully you can embed code in it for more messages.
! You need to find the SubClass proc and add code to handle the WM_SYSCOMMAND/SC_KEYMENU message.
!@ 1. In the frame source search for PROP:WndProc and find the line like: AppFrame{PROP:WndProc} = Address(?NameOfSubClassProc?)
!@ 2. In your proc tree find the procedure named in the above code, i.e. ?NameOfSubClassProc?
!@ 3. Right click on it to view source. It should have some kind of CASE Msg.
!@ 4. Insert the code from step 11 above that is like below.
!@        The _Xxxx variables will have different names, see the Prototype e.g. (SIGNED _hWnd, UNSIGNED _wMsg, UNSIGNED _wParam, SIGNED _LParam)
!@        The Equates will need to be defined in the data section.
!    OF WM_SYSCOMMAND                                        !Win7AltFix  A SysCommand Message, could be the one we want
!       IF BAND(_wParam,0FFF0h)=SC_KEYMENU |                 !Win7AltFix  MSDN says WM_SYSCOMMAND the WParm low 4 bits must be stripped
!       AND _LParam=0 THEN                                   !Win7AltFix _LParam is key pressed, 0=ALT or F10 as done by DefWindowProc
!           PostMessage(_hWnd,WM_SYSCOMMAND,SC_KEYMENU, Alt1stMenu)  !Win7AltFix  Send Alt+F Menu message to drop file menu. FrameHnd,WM_SYSCOMMAND,SC_KEYMENU,"f" key
!           !If you want Alt to not drop the file menu (i.e. do nothing) then do NOT do the PostMessage()
!           RETURN(True)                                     !Win7AltFix  Do NOT pass orig msg on to RTL or it will lock up the frame
!       END
!@ 5. If you cannot embed code in the existing SubClass then it is ok to add another SubClass. Follow the original instrux.
!@       if this causes problems try placing the new Prop:WndProc both before or after the original Prop:WndProc.


! Possible Errors:
!-----------------------------------------------------------------------------------
!    Err: Indistinguishable new prototype: CallWindowProc
!    Why:    CallWindowProc was already defined, delete the new one added to "Inside the Global Map" in step 13
!
!    Err: Indistinguishable new prototype: PostMessage
!    Why:    PostMessage was already defined, delete the new one added to "Inside the Global Map" in step 13
!
!    Err: Unknown Identifier: AppFrame_OrigWndProc
!    Why:    You did not define the variable AppFrame_OrigWndProc LONG in the module or global
!
!    Err:  Unknown Identifier: SubClassFrame
!    Why:    You didn't identify the SubClassFrame procedure
!
!    Err: Unknown Procedure Label: PostMessage   or CallWindowProc
!    Why:    You didn't add the the Global Map declares for PostMessage and CallWindowProc



    INCLUDE 'keycodes.clw'

    map
Main            procedure
ChildWindow     procedure

!#--> 13. Embed Code: Win API "Inside the Global Map" declarations
        MODULE('Win32')            !Win7AltFix
            CallWindowProc(SIGNED lpPrevWndProc,UNSIGNED hWnd,UNSIGNED wMsg,UNSIGNED wParam,SIGNED lParam),SIGNED,RAW,PASCAL,DLL(1),NAME('CallWindowProcA')
            PostMessage(UNSIGNED hWnd,UNSIGNED uMsg,UNSIGNED wParam,LONG lParam),BOOL,PASCAL,PROC,DLL(1),NAME('PostMessageA')
        END                        !Win7AltFix
!#end--Embed Code--------------------------------------------------

SubClassFrame   PROCEDURE(SIGNED _hWnd,UNSIGNED _wMsg,UNSIGNED _wParam,SIGNED _LParam),SIGNED,PASCAL

    end
!#--> 8. Embed Code: To define AppFrame_OrigWndProc LONG as module data
AppFrame_OrigWndProc    LONG    !Win7AltFix
!#end--Embed Code--------------------------------------------------
!In this example the above LONG is Global. I suggest you make this a Module variable and put the SubClass proc into the
!same module as Main. The only different is it keeps it out of scope of all the procedures that don't need it.
!Hopefully an entire Module would be swapped into memory so having the 2 procs together would be an advantage
!Need to look into the order the procs are placed by the IDE. If it is Alphabetical then naming them accordingly would help
!keep them closer in memory

    code
    Main()
    return
    
!------------------------------------- Main (Frame) ---------------------------------------------------------------------
Main    procedure
RemoveAltFix    BOOL

AppFrame APPLICATION('Win 7 Alt Lockup Fix by Carl Barnes'),AT(,,471,274),FONT('MS Sans Serif',8,,FONT:regular), |
         CENTER,ICON(Icon:ThumbNail),SYSTEM,RESIZE
       MENUBAR
         MENU('&File'),USE(?File),KEY(f7key)
           ITEM('Start MDI Child Window'),USE(?ClientWindow),KEY(f8key)
           ITEM,SEPARATOR
           ITEM('Printer Setup'),USE(?Prn),STD(STD:PrintSetup)
           ITEM,SEPARATOR
           ITEM('E&xit'),USE(?Exit),LAST,STD(STD:Close)
         END
         MENU('&Edit'),USE(?Edit)
           ITEM('Cu&t<9>Ctrl+X'),USE(?Cut),STD(STD:Cut)
           ITEM('&Copy<9>Ctrl+C'),USE(?Copy),STD(STD:Copy)
           ITEM('&Paste<9>Ctrl+V'),USE(?Paste),STD(STD:Paste)
         END
         MENU('&Window'),LAST,STD(STD:WindowList)
           ITEM('Tile &Horizontal'),USE(?TileHor),STD(STD:TileHorizontal)
           ITEM('Tile &Vertical'),USE(?WindowTileVertical),STD(STD:TileVertical)
           ITEM('&Cascade'),USE(?Cascade),STD(STD:CascadeWindow)
           ITEM('&Arrange Icons'),USE(?Arrange),STD(STD:ArrangeIcons)
         END
         MENU('&Help')
           ITEM('Help'),USE(?HelpStd),STD(STD:Help)
         END
       END
       TOOLBAR,AT(,,,14)
         BUTTON('Start Child'),AT(1,1,50,10),USE(?CLientFrame),FLAT,LEFT,TIP('Start new Child Window in Thread')
         BUTTON('Close'),AT(51,1,37,10),USE(?BreakFrame),FLAT,LEFT,TIP('Cioa')
         CHECK('Remove Alt Fix'),AT(248,2),USE(RemoveAltFix),SKIP,TIP('Unhook the Subclass procedure to remove the Alt fix<13,10>Press and release Alt or F1' &|
             '0 to see lockup<13,10>Press Alt+Esc to unlock')
         STRING(' Press Alt+Esc to unlock '),AT(312,3,,8),USE(?String_RemoveAltFix),HIDE,FONT(,,COLOR:Blue,,CHARSET:ANSI), |
             COLOR(COLOR:White)
       END
     END

    CODE

    OPEN(AppFrame)
    AppFrame{PROP:TExt}=clip(AppFrame{PROP:TExt}) & ' - Clarion ' & clip(SYSTEM{PROP:LibVersion}) &' / '&  clip(SYSTEM{PROP:ExeVersion}) !&' version of CW Library'

!#--> 5. Embed Code: To Subclass the frame with a Prop:WndProc reassignment
    AppFrame_OrigWndProc   = AppFrame{PROP:WndProc}             !Win7AltFix
    AppFrame{PROP:WndProc} = Address(SubClassFrame)             !Win7AltFix
!#end--Embed Code--------------------------------------------------
    accept
        case event()
        of event:OpenWindow                 ;   POST(event:accepted,?ClientWindow)
        end
        case accepted()
        of ?ClientWindow orof ?CLientFrame  ;   START(ChildWindow)
        of ?BreakFrame                      ;   POST(event:CloseDown)
        of ?RemoveAltFix
            if RemoveAltFix                                     
               AppFrame{PROP:WndProc} = AppFrame_OrigWndProc    !no subclass, let it lockup
               unhide(?String_RemoveAltFix)
            else
               AppFrame{PROP:WndProc} = Address(SubClassFrame)
               hide(?String_RemoveAltFix)
            end
        end
    end
    close(AppFrame)
    return


!-------------------------------------- ChildWindow (Window) --------------------------------------------------------------------
ChildWindow    procedure

Entry1      string(20)
Entry2      string(20)
Entry3      string(20)
Entry4      string(255)

AlertF10Btn     long
AlertAltFBtn    long
AlertAltOnly    long

ChWindow WINDOW('Child Window'),AT(,,207,124),FONT('Verdana',8,,),ICON(icon:help),SYSTEM,GRAY,DOUBLE, |
         MDI
       PROMPT('E&ntry 1'),AT(7,4),USE(?Prompt3)
       ENTRY(@s20),AT(40,4,89,10),USE(Entry1)
       PROMPT('&Entry 2'),AT(7,16),USE(?Prompt4)
       ENTRY(@s20),AT(40,16,89,10),USE(Entry2)
       PROMPT('Ent&ry 3'),AT(7,28),USE(?Prompt5)
       ENTRY(@s20),AT(40,28,89,10),USE(Entry3)
       PROMPT('Entry &4:'),AT(7,40),USE(?Prompt:Topic)
       ENTRY(@s255),AT(40,40,123,10),USE(Entry4)
       BUTTON('&Close'),AT(25,58,51,12),USE(?CloseBtn),STD(STD:Close)
       BUTTON('&Open Child'),AT(87,58,51,12),USE(?OpenChildBtn),TIP('Open same child window, do not START()<13,10>So can test Alerts do not affect child b' &|
           'eing fixed')
       CHECK('Alert F&10'),AT(7,79,51,12),USE(AlertF10Btn),TIP('F10 also activates menu and hangs<13,10>But it is also used to locally<13,10>Push button f' &|
           'or local use')
       STRING('Check box to alert F10'),AT(78,81),USE(?AlertMsgF10)
       CHECK('Alert &Alt+F'),AT(7,90,51,12),USE(AlertAltFBtn),TIP('An &F hot key on the local window will grab the one I want to use to open the fr' &|
           'ame')
       STRING('Check box to alert Alt+F'),AT(78,92),USE(?AlertMsgAltF)
       CHECK('Alert A&LT only (Template Fix)'),AT(7,110),USE(AlertAltOnly),TIP('Alerts ALT key (ALERT(1024) to test what happens with template type fix<13,10>Menu ke' &|
           'y underlines may not appear<13,10>Alt+ Key Menu access may not work')
     END

    code
    open(ChWindow)
    0{prop:text}=clip(0{prop:text})&' Thread' & Thread()
    accept
        case event()
        of event:AlertKey
            case keycode()
            of F10key   ;  ?AlertMsgF10{Prop:Text}='Pressed F10 at ' & clock()
            of AltF     ;  ?AlertMsgAltF{Prop:Text}='Pressed Alt+F at ' & clock()
            of 1024     !Alt key released, do nothing
            end
        end
        case accepted()
        of ?OpenChildBtn  ;           ChildWindow()
        of ?AlertF10Btn   ;         ALERT(F10Key); disable(?AlertF10Btn) ; ?AlertMsgF10{Prop:Text}='Press F10, see this change'
        of ?AlertAltFBtn  ;         ALERT(AltF)  ; disable(?AlertAltFBtn) ; ?AlertMsgAltF{Prop:Text}='Press Alt+F or ALT with fix'
        of ?AlertAltOnly  ;         ALERT(1024)  ; disable(?AlertAltOnly) ; ?AlertAltOnly{Prop:Text}=CLIP(?AlertAltOnly{Prop:Text})&' - ALT now does nothing'
        end
    end
    close(ChWindow)

    !Todo: Make the checkboxes uncheckable. Would need to assign with 0{PROP:ALRT,255}= then to UNalert search for 0{PROP:ALRT,#}=key
    !       For now just start a new child window.

!You could do the fix for just Windows 7, but if you use GetVersionEx() it will not return Win 7 if the user has shortcut set
!to Compatibility of XP or Vista, but he's still running Win 7 and will lockup. One way to spot Win 7 is to look for a
!new function like SetWaitableTimerEXProc(), i.e. GetModuleHandle('Kernel32.dll') then GetProcAddress('SetWaitableTimerEXProc').
!I think it is safe to do the fix for all NT versions. I have not tested 9x but would expect it to work.
!-------------------------------------- SubClass --------------------------------------------------------------------
SubClassFrame       PROCEDURE(SIGNED _hWnd, UNSIGNED _wMsg, UNSIGNED _wParam, SIGNED _LParam)!,SIGNED,PASCAL !Win7AltFix
!#--> 11. Embed Code: To define the Subclass procedure code. If &File is not 1st menu then change Alt1stMenu EQUATE.
WM_SYSCOMMAND       EQUATE(0112h)       !WM_SYSCOMMAND 0x0112   !Win7AltFix
SC_KEYMENU          EQUATE(0F100h)      !SC_KEYMENU 0xF100      !Win7AltFix
!SC_AndMask         EQUATE(0FFF0h)      !WM_SYSCOMMAND msgs the 4 low-order bits of wParam are used internally, you must strip lower 4 bits to compare to SC_xxx
Alt1stMenu          EQUATE(val('f')) !<<< set to your first menu Alt letter, i.e. &File is Alt+f (use lower case)
!?>?>?>?>?>?>?>?>?>?  is MENU('&File') your first menu ????? Adjust for Your Menu ???????????????????????????????????????????????????????
  CODE
    CASE _wMsg                                              !Win7AltFix 
    OF WM_SYSCOMMAND                                        !Win7AltFix  A SysCommand Message, could be the one we want
       IF BAND(_wParam,0FFF0h)=SC_KEYMENU |                 !Win7AltFix  MSDN says WM_SYSCOMMAND the WParm low 4 bits must be stripped
       AND _LParam=0 THEN                                   !Win7AltFix _LParam is key pressed, 0=ALT or F10 as done by DefWindowProc
           PostMessage(_hWnd,WM_SYSCOMMAND,SC_KEYMENU, Alt1stMenu)  !Win7AltFix  Send Alt+F Menu message to drop file menu. FrameHnd,WM_SYSCOMMAND,SC_KEYMENU,"f" key
           !If you want Alt to not drop the file menu (i.e. do nothing) then do NOT do the PostMessage()
           RETURN(True)                                     !Win7AltFix  Do NOT pass orig msg on to RTL or it will lock up the frame
       END

    !--If you would like your App to shutdown with Windows add the below 2 lines
        OF 0011h   ;      RETURN(True)      !WM_QUERYENDSESSION  EQUATE(0011h)
        OF 0016h   ;      RETURN(True)      !WM_ENDSESSION       EQUATE(0016h)
    !--If you would like your App to shutdown with Windows add the above 2 lines

    END  !CASE _wMsg                                                          !Win7AltFix 
    RETURN(CallWindowProc(AppFrame_OrigWndProc,_hWnd,_wMsg,_wParam,_LParam))  !Win7AltFix 
!#end--#--> 11. Embed Code--------------------------------------------------

!----------------------------------------------------------------------------------------------------------
! I tested this type of code that changed _LParam to F and then let that go onto the Clarion menu
!
!       AND _LParam=0 THEN                    !  _LParam is key pressed, 0=ALT or F10 as done by DefWindowProc
!           _LParam = val('f')                !Send an Alt+F to drop File menu, you do have a file menu?
!           !do NOT do a RETURN(True), let the message changed to Alt+F go on to Clarion menu
!    END  !CASE _wMsg
!    RETURN(CallWindowProc(AppFrame_OrigWndProc,_hWnd,_wMsg,_wParam,_LParam))
!
! It did work, except on one system...and maybe it was fine on that system.
! I felt it was a bit trickey and decided that posting my own Drop the menu message was safer
!----------------------------------------------------------------------------------------------------------







!==Quick-Start Instructions===================================================================
! Just the code and some very quick instructions
  OMIT('!-- End Quick-Start --')
!#--> 1. Open the .App file with your Frame (Main)
!#--> 5. Open the Frame, find Open(AppFrame) and after it Embed Code to Subclass the frame

    AppFrame_OrigWndProc   = AppFrame{PROP:WndProc}             !Win7AltFix
    AppFrame{PROP:WndProc} = Address(SubClassFrame)             !Win7AltFix

!#--> 8. Define AppFrame_OrigWndProc LONG as module data or global

AppFrame_OrigWndProc    LONG    !Win7AltFix

!#--> 10. Add a new SOURCE procedure "SubClassFrame" to the same Module as the Frame (Main)
!#-->       Prototype:  (SIGNED _hWnd,UNSIGNED _wMsg,UNSIGNED _wParam,SIGNED _LParam),SIGNED,PASCAL   !Win7AltFix
!#-->       Parameters: (SIGNED _hWnd,UNSIGNED _wMsg,UNSIGNED _wParam,SIGNED _LParam)!!,SIGNED,PASCAL !Win7AltFix
!#--> 11. In "SubClassFrame" Embed Code. If &File is not 1st menu then change Alt1stMenu EQUATE.

SubClassFrame       PROCEDURE(SIGNED _hWnd, UNSIGNED _wMsg, UNSIGNED _wParam, SIGNED _LParam)!,SIGNED,PASCAL !Win7AltFix
WM_SYSCOMMAND       EQUATE(0112h)       !WM_SYSCOMMAND 0x0112   !Win7AltFix
SC_KEYMENU          EQUATE(0F100h)      !SC_KEYMENU 0xF100      !Win7AltFix
!SC_AndMask         EQUATE(0FFF0h)      !WM_SYSCOMMAND msgs the 4 low-order bits of wParam are used internally, you must strip lower 4 bits to compare to SC_xxx
Alt1stMenu          EQUATE(val('f')) !<<< set to your first menu Alt letter, i.e. &File is Alt+f (use lower case)
!?>?>?>?>?>?>?>?>?>?  is MENU('&File') your first menu ????? Adjust for Your Menu ???????????????????????????????????????????????????????
  CODE
    CASE _wMsg                                              !Win7AltFix 
    OF WM_SYSCOMMAND                                        !Win7AltFix  A SysCommand Message, could be the one we want
       IF BAND(_wParam,0FFF0h)=SC_KEYMENU |                 !Win7AltFix  MSDN says WM_SYSCOMMAND the WParm low 4 bits must be stripped
       AND _LParam=0 THEN                                   !Win7AltFix _LParam is key pressed, 0=ALT or F10 as done by DefWindowProc
           PostMessage(_hWnd,WM_SYSCOMMAND,SC_KEYMENU, Alt1stMenu)  !Win7AltFix  Send Alt+F Menu message to drop file menu. FrameHnd,WM_SYSCOMMAND,SC_KEYMENU,"f" key
           !If you want Alt to not drop the file menu (i.e. do nothing) then do NOT do the PostMessage()
           RETURN(True)                                     !Win7AltFix  Do NOT pass orig msg on to RTL or it will lock up the frame
       END
    END  !CASE _wMsg                                                          !Win7AltFix
    RETURN(CallWindowProc(AppFrame_OrigWndProc,_hWnd,_wMsg,_wParam,_LParam))  !Win7AltFix 


!#--> 12. Add Win API Declarations. Open Global, then Embeds, then "Inside the Global Map" embed point

        MODULE('Win32')            !Win7AltFix
            CallWindowProc(SIGNED lpPrevWndProc,UNSIGNED hWnd,UNSIGNED wMsg,UNSIGNED wParam,SIGNED lParam),SIGNED,RAW,PASCAL,DLL(1),NAME('CallWindowProcA')
            PostMessage(UNSIGNED hWnd,UNSIGNED uMsg,UNSIGNED wParam,LONG lParam),BOOL,PASCAL,PROC,DLL(1),NAME('PostMessageA')
        END                        !Win7AltFix

!-- End Quick-Start --------------------------------------------------------------------------
